@@ -51,9 +51,6 @@ static int      timer_running = 0;
 static int      soundcards_installed = 0;	/* Number of installed
 						 * soundcards */
 static int      soundcard_configured = 0;
-extern char    *snd_raw_buf[MAX_DSP_DEV][DSP_BUFFCOUNT];
-extern unsigned long snd_raw_buf_phys[MAX_DSP_DEV][DSP_BUFFCOUNT];
-extern int      snd_raw_count[MAX_DSP_DEV];
 
 static struct fileinfo files[SND_NDEVS];
 
@@ -80,7 +77,7 @@ get_time(void)
   return timecopy.tv_usec/(1000000/HZ) +
 	  (unsigned long)timecopy.tv_sec*HZ;
 }
- 
+
 int
 sndread (int dev, struct uio *buf)
 {
@@ -185,7 +182,6 @@ int
 sndattach (struct isa_device *dev)
 {
   int             i;
-  static int      dsp_initialized = 0;
   static int      midi_initialized = 0;
   static int      seq_initialized = 0;
   static int 	  generic_midi_initialized = 0; 
@@ -216,37 +212,21 @@ sndattach (struct isa_device *dev)
   printf("\n");
 
 #ifndef EXCLUDE_AUDIO
-  soundcard_configured = 1;
-  if (num_dspdevs)
-    sound_mem_init ();
-#endif
-
-  if (num_dspdevs && !dsp_initialized)	/* Audio devices present */
+  if (num_audiodevs)	/* Audio devices present */
     {
-      dsp_initialized = 1;
       mem_start = DMAbuf_init (mem_start);
       mem_start = audio_init (mem_start);
+      sound_mem_init ();
     }
 
-/** UWM stuff **/
+  soundcard_configured = 1;
+#endif
 
-#ifndef EXCLUDE_CHIP_MIDI
-
-     if (!generic_midi_initialized)
-     {
-	 generic_midi_initialized = 1;
-	 mem_start = CMIDI_init (mem_start);
-     } 
-
-#endif 
-
-#ifndef EXCLUDE_MPU401
   if (num_midis && !midi_initialized)
     {
       midi_initialized = 1;
       mem_start = MIDIbuf_init (mem_start);
     }
-#endif
 
   if ((num_midis + num_synths) && !seq_initialized)
     {
@@ -266,14 +246,6 @@ tenmicrosec (void)
     inb (0x80);
 }
 
-#ifdef EXCLUDE_GUS
-void
-gusintr (int unit)
-{
-  return (0);
-}
-#endif
-
 void
 request_sound_timer (int count)
 {
@@ -316,7 +288,7 @@ sound_mem_init (void)
   unsigned long   dma_pagesize;
   static unsigned long dsp_init_mask = 0;
 
-  for (dev = 0; dev < num_dspdevs; dev++)	/* Enumerate devices */
+  for (dev = 0; dev < num_audiodevs; dev++)	/* Enumerate devices */
     if (!(dsp_init_mask & (1 << dev)))	/* Not already done */
       if (sound_buffcounts[dev] > 0 && sound_dsp_dmachan[dev] > 0)
 	{
@@ -350,9 +322,9 @@ sound_mem_init (void)
 	  if (sound_buffsizes[dev] < 4096)
 	    sound_buffsizes[dev] = 4096;
 #else
-	dma_pagesize = 4096;
-	sound_buffsizes[dev] = 4096;
-	sound_buffcounts[dev] = 16;	/* 16*4k -> 64k */
+	  dma_pagesize = 4096;
+	  sound_buffsizes[dev] = 4096;
+	  sound_buffcounts[dev] = 16;	/* 16*4k -> 64k */
 #endif
 
 	  /* Now allocate the buffers */
@@ -367,7 +339,7 @@ sound_mem_init (void)
 	       * This really needs some kind of finetuning.
 	       */
 	      char           *tmpbuf = malloc (2*sound_buffsizes[dev], M_DEVBUF, M_NOWAIT);
-	      unsigned long   addr, rounded;
+	      unsigned long   addr, rounded, start, end;
 
 	      if (tmpbuf == NULL)
 		{
@@ -378,9 +350,15 @@ sound_mem_init (void)
 
 	      addr = kvtop (tmpbuf);
 	      /*
-	       * Align the start address
+	       * Align the start address if required
 	       */
-	      rounded = (addr & ~(dma_pagesize - 1)) + dma_pagesize;
+	      start = (addr & ~(dma_pagesize - 1));
+	      end = ((addr+sound_buffsizes[dev]-1) & ~(dma_pagesize - 1));
+
+	      if (start != end)
+	         rounded = end;
+	      else
+                 rounded = addr;	/* Fits to the same DMA page */
 
 	      snd_raw_buf[dev][snd_raw_count[dev]] =
 		&tmpbuf[rounded - addr];	/* Compute offset */
